import fs from 'fs';

const directory = './src/lib/components';
const directoryReplace = '$lib/components';
const directoryIndex = './src/lib/components/index.ts';
const componentsDirectory = ['atom', 'molecules', 'organisms', 'pages', 'templates'];

const notIncludes = ['components/icons'];

const files = (directory) => {
	const directoryFiles = fs.readdirSync(directory);

	return directoryFiles.map(file => {
		const currentDirectory = `${directory}/${file}`;
		if (notIncludes.findIndex(i => currentDirectory.includes(i)) > -1 || file.indexOf("index.ts") > -1) return;
		const stat = fs.statSync(currentDirectory);

		if (stat.isDirectory()) {
			return files(currentDirectory);
		}

		return { key: currentDirectory.split('/').at(-1).replace('.svelte', ''), file: currentDirectory };
	});
};
const firstUppercase = (word) => word.charAt(0).toUpperCase() + word.slice(1);
const transformJson = (files) => {
	let jsonImports = {};

	files.map(([key, value], index) => {
		jsonImports[firstUppercase(key)] = value;
	});

	return jsonImports;
};
const generateImports = (files, indexFile) => {
	let importsFiles = [];
	let exportsFiles = {};

	Object.entries(files).forEach(([key, value]) => {
		value.forEach(({ key: key2, file }) => {
			importsFiles.push(`import ${key}${key2} from '${file.replace(directory, directoryReplace)}'`);
			(exportsFiles[key] ?? (exportsFiles[key] = [])).push(`${key}${key2}`);
		});
	});

	return [importsFiles, exportsFiles];
};

const generateOtherExport = (key, value) => {
	return value.reduce((total, val) => ([...total, `\t${val},\n`]), []).join('');
};

const saveImports = (template, fileSave) => {
	return fs.writeFileSync(fileSave, template);
};

const generateTemplate = (imports, exports) => {
	let text = '';

	text = imports.join(';\n');
	text += '\n';

	text += 'export {\n';

	text += Object.entries(exports).reduce((total, [key, value]) => {
		return [...total, `${generateOtherExport(key, value)}`];
	}, []).join('');

	text += '}';
	return text;
};

console.log('--- GENERATE COMPONENTS---');
const [imports, exports] = generateImports(transformJson(componentsDirectory.map(i => [i, files([directory, i].join('/')).flat(2)])), directoryIndex);
const template = generateTemplate(imports, exports);
saveImports(template, directoryIndex);
console.log('--- END GENERATE COMPONENTS---');
