import { browser } from '$app/environment';
import { writable } from 'svelte/store';

export const writableStorage = <T>(key: string, initialValue: T) => {
	const initialValueCache = (): T => {
		const value = localStorage.getItem(key);
		if (!value) return initialValue;
		return JSON.parse(value).data;
	};

	const value = writable<T>(browser ? initialValueCache() : initialValue);

	value.subscribe(e => {
		if (!browser) return;
		localStorage.setItem(key, JSON.stringify({ data: e }));
	});

	return value;
};
